@@ -1,5 +1,8 @@
-binomial.independent <- function(formula, data=NULL, beta=NULL, theta=NULL, sigma2=NULL, trials, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, blocksize.theta=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.sigma2=NULL)
+binomial.independent <- function(formula, data=NULL, beta=NULL, theta=NULL, sigma2=NULL, trials, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, prior.mean.beta=NULL, prior.var.beta=NULL, prior.sigma2=NULL)
 {
+cat("Setting up the model\n")
+a<-proc.time()
+
 ##############################################
 #### Format the arguments and check for errors
 ##############################################
@@ -58,7 +61,8 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
 #### Response variable and trials
 ## Create the response
 Y <- model.response(frame)
-    
+failures <- trials - Y
+     
 ## Check for errors
     if(sum(is.na(trials))>0) stop("the numbers of trials has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(trials)) stop("the numbers of trials has non-numeric values.", call.=FALSE)
@@ -89,7 +93,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 
 #### Initial parameter values
 ## Regression parameters beta
-dat <- cbind(Y, trials-Y)
+dat <- cbind(Y, failures)
 
     if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
     if(length(beta)!= p) stop("beta is the wrong length.", call.=FALSE)
@@ -110,6 +114,28 @@ dat <- cbind(Y, trials-Y)
     if(sigma2 <= 0) stop("sigma2 is negative or zero.", call.=FALSE)
 
 
+#### Priors
+## Put in default priors
+    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.sigma2)) prior.sigma2 <- c(0.001, 0.001)
+     
+## Checks    
+    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
+ 
+    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
+    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
+
+    if(length(prior.sigma2)!=2) stop("the prior value for sigma2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.sigma2)) stop("the prior value for sigma2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.sigma2))!=0) stop("the prior value for sigma2 has missing values.", call.=FALSE)    
+
+     
+     
 #### MCMC quantities
 ## Checks
     if(!is.numeric(burnin)) stop("burn-in is not a number", call.=FALSE)
@@ -123,12 +149,9 @@ dat <- cbind(Y, trials-Y)
     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.beta)==ceiling(blocksize.beta))) stop("blocksize.beta has non-integer values.", call.=FALSE)
-    if(!is.numeric(blocksize.theta)) stop("blocksize.theta is not a number", call.=FALSE)
-    if(blocksize.theta <= 0) stop("blocksize.theta is less than or equal to zero", call.=FALSE)
-    if(!(floor(blocksize.theta)==ceiling(blocksize.theta))) stop("blocksize.theta has non-integer values.", call.=FALSE)
 
 
-## Compute the blocking structure for beta
+## Compute the blocking structure for beta     
      if(blocksize.beta >= p)
      {
      n.beta.block <- 1
@@ -142,45 +165,17 @@ dat <- cbind(Y, trials-Y)
           if(remainder==0)
           {
           beta.beg <- c(1,seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta))
+          beta.fin <- seq(blocksize.beta, p, blocksize.beta)
           n.beta.block <- length(beta.beg)
           }else
           {
           beta.beg <- c(1, seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta), p)
+          beta.fin <- c(seq((blocksize.beta), p, blocksize.beta), p)
           n.beta.block <- length(beta.beg)
           }
-     }         
+     }           
 
 
-## Compute the blocking structure for theta
-     if(blocksize.theta >= n)
-     {
-     n.theta.block <- 1
-     theta.beg <- 1
-     theta.fin <- n  
-     }else
-     {
-     n.standard <- 1 + floor((n-blocksize.theta) / blocksize.theta)
-     remainder <- n - (n.standard * blocksize.theta)
-     
-          if(remainder==0)
-          {
-          theta.beg <- c(1,seq((blocksize.theta+1), n, blocksize.theta))
-          theta.fin <- c(blocksize.theta, seq((blocksize.theta+blocksize.theta), n, blocksize.theta))
-          n.theta.block <- length(theta.beg)
-          }else if(remainder==1)
-          {
-          theta.beg <- c(1, seq((blocksize.theta), n, blocksize.theta))
-          theta.fin <- c(blocksize.theta-1, seq((blocksize.theta+blocksize.theta-1), n, blocksize.theta), n)
-          n.theta.block <- length(theta.beg)    
-          }else
-          {
-          theta.beg <- c(1, seq((blocksize.theta+1), n, blocksize.theta))
-          theta.fin <- c(blocksize.theta, seq((blocksize.theta+blocksize.theta), n, blocksize.theta), n)
-          n.theta.block <- length(theta.beg)
-          }
-     }
 
 
 ## Matrices to store samples
@@ -197,66 +192,30 @@ proposal.sd.beta <- 0.01
 proposal.sd.theta <- 0.01
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
 chol.proposal.corr.beta <- chol(proposal.corr.beta) 
-sigma2.posterior.shape <- 0.5 * n - 1
-
-
-#### Priors
-## Put in default priors
-## N(0, 100) for beta 
-## U(0, 10) for sigma2
-    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
-    if(is.null(prior.max.sigma2)) prior.max.sigma2 <- 1000
-
-## Checks    
-    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
- 
-    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
-    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
-
-    if(length(prior.max.sigma2)!=1) stop("the maximum prior value for sigma2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.sigma2)) stop("the maximum prior value for sigma2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.sigma2))!=0) stop("the maximum prior value for sigma2 has missing values.", call.=FALSE)    
-    if(min(prior.max.sigma2) <=0) stop("the maximum prior value for sigma2 is less than zero", call.=FALSE)
-
-
-
-#### Other quantities needed for the MCMC algorithm
-failures <- trials - Y
-
+sigma2.posterior.shape <- prior.sigma2[1] + 0.5 * n
 
 
 ###########################
 #### Run the Bayesian model
 ###########################
-    for(j in 1:n.sample)
+## Start timer
+cat("Collecting", n.sample, "samples\n", sep = " ")
+progressBar <- txtProgressBar(style = 3)
+percentage.points<-round((1:100/100)*n.sample)
+
+     for(j in 1:n.sample)
     {
     ####################
     ## Sample from beta
     ####################
     proposal <- beta + (sqrt(proposal.sd.beta)* t(chol.proposal.corr.beta)) %*% rnorm(p)
-    proposal.beta <- beta    
-    theta.offset <- theta + offset
-    
-        for(r in 1:n.beta.block)
-        {
-        ## Propose a value
-        proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-        logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + theta.offset
-        logit.current <- as.numeric(X.standardised %*% beta) + theta.offset    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-             
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y * (log(prob.proposal) - log(prob.current)) + failures * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- sum(((beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2 - (proposal.beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2) / (2 * prior.var.beta[beta.beg[r]:beta.fin[r]]))
-        prob <- exp(prob1 + prob2)
+    proposal.beta <- beta
+    offset.temp <- theta + offset
 
-        ## Accept or reject the value
+       for(r in 1:n.beta.block)
+       {
+       proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
+       prob <- binomialbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta)
             if(prob > runif(1))
             {
             beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -266,46 +225,28 @@ failures <- trials - Y
             proposal.beta[beta.beg[r]:beta.fin[r]] <- beta[beta.beg[r]:beta.fin[r]]
             }
         }
-    accept[2] <- accept[2] + n.beta.block
+
+    accept[2] <- accept[2] + n.beta.block    
+ 
 
 
     ####################
     ## Sample from theta
     ####################
-    proposal.theta <- rnorm(n, mean=theta, sd=proposal.sd.theta)
     beta.offset <- as.numeric(X.standardised %*% beta) + offset        
-    logit.proposal <- beta.offset + proposal.theta
-    logit.current <- beta.offset + theta    
-    prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-    prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-    
-        for(r in 1:n.theta.block)
-        {
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y[theta.beg[r]:theta.fin[r]] * (log(prob.proposal[theta.beg[r]:theta.fin[r]]) - log(prob.current[theta.beg[r]:theta.fin[r]])) + failures[theta.beg[r]:theta.fin[r]] * (log(1-prob.proposal[theta.beg[r]:theta.fin[r]]) - log(1-prob.current[theta.beg[r]:theta.fin[r]])))          
-	   prob2 <- sum(theta[theta.beg[r]:theta.fin[r]]^2  - proposal.theta[theta.beg[r]:theta.fin[r]]^2) / sigma2
-        prob <- exp(prob1 + 0.5 * prob2)
-
-        ## Accept or reject the value
-            if(prob > runif(1))
-            {
-            theta[theta.beg[r]:theta.fin[r]] <- proposal.theta[theta.beg[r]:theta.fin[r]]
-            accept[3] <- accept[3] + 1  
-            }else
-            {
-            }
-        }
-    accept[4] <- accept[4] + n.theta.block
-    theta <- theta - mean(theta)
-    
-    
+    temp1 <- binomialindepupdate(nsites=n, theta=theta, sigma2=sigma2, y=Y, failures=failures, theta_tune=proposal.sd.theta, offset=beta.offset) 
+    theta <- temp1[[1]]
+    theta <- theta - mean(theta)    
+    accept[3] <- accept[3] + temp1[[2]]
+    accept[4] <- accept[4] + n
+  
+          
     
     ####################
     ## Sample from sigma2
     ####################
-    sigma2.posterior.scale <- 0.5*sum(theta^2)
-    sigma2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.sigma2), b=Inf,  shape=sigma2.posterior.shape, scale=(1/sigma2.posterior.scale))
-    
+    sigma2.posterior.scale <- prior.sigma2[2] + 0.5*sum(theta^2)
+    sigma2 <- 1 / rgamma(1, sigma2.posterior.shape, scale=(1/sigma2.posterior.scale))    
             
     
     #########################
@@ -371,21 +312,18 @@ failures <- trials - Y
 
     
     
-    #######################################
-    #### Print out the number of iterations
-    #######################################
-    k <- j/1000
-        if(ceiling(k)==floor(k))
-        {
-        cat("Completed ",j, " samples\n")
-        flush.console()
-        }else
-        {
-        }
-}
-
-
+    ################################       
+    ## print progress to the console
+    ################################
+          if(j %in% percentage.points)
+          {
+          setTxtProgressBar(progressBar, j/n.sample)
+          }
+     }
 
+# end timer
+cat("\nSummarising results")
+close(progressBar)
 ###################################
 #### Summarise and save the results 
 ###################################
@@ -407,7 +345,18 @@ deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE)
 p.d <- mean(samples.deviance) - deviance.fitted
 DIC <- 2 * mean(samples.deviance) - deviance.fitted
 
+#### Compute the Conditional Predictive Ordinate
+CPO.temp <- array(NA, c(nrow(samples.theta), n))
+    for(i in 1:nrow(samples.theta))
+    {
+    temp.logit <- samples.theta[i, ]  + X.standardised %*% samples.beta[i, ] + offset
+    temp <- exp(temp.logit)  / (1 + exp(temp.logit))
+    CPO.temp[i, ] <- 1 / dbinom(x=Y, size=trials, prob=temp)
+    }
+CPO <- 1/apply(CPO.temp, 2, mean)
+MPL <- sum(log(CPO))     
 
+     
 
 #### transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- samples.beta
@@ -489,8 +438,10 @@ residuals <- round(residuals, 4)
 ## Compile and return the results
 model.string <- c("Likelihood model - Binomial (logit link function)", "\nRandom effects model - Independent\n")
 samples <- list(beta=samples.beta.orig, theta=mcmc(samples.theta), sigma2=mcmc(samples.sigma2))
-results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=NULL, DIC=DIC, p.d=p.d, summary.results=summary.results, model=model.string, accept=accept.final)
+results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=NULL, DIC=DIC, p.d=p.d, MPL=MPL, summary.results=summary.results, model=model.string, accept=accept.final)
 class(results) <- "carbayes"
+b<-proc.time()
+cat(" finished in ", round(b[3]-a[3], 1), "seconds")
 return(results)
 }
 
