@@ -1,6 +1,9 @@
 binomial.lerouxCAR <-
-function(formula, data=NULL, beta=NULL, phi=NULL, tau2=NULL, rho=NULL, trials, W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, blocksize.phi=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.tau2=NULL)
+function(formula, data=NULL, beta=NULL, phi=NULL, tau2=NULL, rho=NULL, trials, W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, prior.rho=NULL)
 {
+cat("Setting up the model\n")
+a<-proc.time()
+
 ##############################################
 #### Format the arguments and check for errors
 ##############################################
@@ -59,7 +62,8 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
 #### Response variable and trials
 ## Create the response
 Y <- model.response(frame)
-    
+failures <- trials - Y
+     
 ## Check for errors
     if(sum(is.na(trials))>0) stop("the numbers of trials has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(trials)) stop("the numbers of trials has non-numeric values.", call.=FALSE)
@@ -90,7 +94,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 
 #### Initial parameter values
 ## Regression parameters beta
-dat <- cbind(Y, trials-Y)
+dat <- cbind(Y, failures)
 
     if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
     if(length(beta)!= p) stop("beta is the wrong length.", call.=FALSE)
@@ -111,14 +115,40 @@ dat <- cbind(Y, trials-Y)
     if(tau2 <= 0) stop("tau2 is negative or zero.", call.=FALSE)
 
 ## Global correlation parameter rho
-    if(is.null(rho)) rho <- runif(1)
+    if(is.null(prior.rho)) prior.rho <- seq(0, 0.99, 0.01)
+    if(min(prior.rho) < 0 | max(prior.rho) >=1) stop("prior.rho is outside the interval [0,1).", call.=FALSE)
+    if(sum(duplicated(prior.rho))>0) stop("prior.rho has duplicate values.", call.=FALSE)     
+    if(is.null(rho)) rho <- sample(prior.rho, size=1)
+    if(sum(rho==prior.rho)!= 1) stop("rho is not one of the possible values.", call.=FALSE)
     if(length(rho)!= 1) stop("rho is the wrong length.", call.=FALSE)
     if(sum(is.na(rho))>0) stop("rho has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(rho)) stop("rho has non-numeric values.", call.=FALSE)
     if(rho < 0 | rho >=1) stop("rho is outside the interval [0,1).", call.=FALSE)
+which.rho <- which(rho==prior.rho)    
     
-    
+#### Priors
+## Put in default priors
+    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.tau2)) prior.tau2 <- c(0.001, 0.001)
+ 
+     
+## Checks    
+    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
+ 
+    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
+    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
 
+    if(length(prior.tau2)!=2) stop("the prior value for tau2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.tau2)) stop("the prior value for tau2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.tau2))!=0) stop("the prior value for tau2 has missing values.", call.=FALSE)    
+
+     
+     
 #### MCMC quantities
 ## Checks
     if(!is.numeric(burnin)) stop("burn-in is not a number", call.=FALSE)
@@ -132,12 +162,9 @@ dat <- cbind(Y, trials-Y)
     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.beta)==ceiling(blocksize.beta))) stop("blocksize.beta has non-integer values.", call.=FALSE)
-     if(!is.numeric(blocksize.phi)) stop("blocksize.phi is not a number", call.=FALSE)
-    if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
-    if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
+ 
 
-
-## Compute the blocking structure for beta
+## Compute the blocking structure for beta     
      if(blocksize.beta >= p)
      {
      n.beta.block <- 1
@@ -151,45 +178,15 @@ dat <- cbind(Y, trials-Y)
           if(remainder==0)
           {
           beta.beg <- c(1,seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta))
+          beta.fin <- seq(blocksize.beta, p, blocksize.beta)
           n.beta.block <- length(beta.beg)
           }else
           {
           beta.beg <- c(1, seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta), p)
+          beta.fin <- c(seq((blocksize.beta), p, blocksize.beta), p)
           n.beta.block <- length(beta.beg)
           }
-     }         
-
-
-## Compute the blocking structure for phi
-     if(blocksize.phi >= n)
-     {
-     n.phi.block <- 1
-     phi.beg <- 1
-     phi.fin <- n  
-     }else
-     {
-     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
-     remainder <- n - (n.standard * blocksize.phi)
-     
-          if(remainder==0)
-          {
-          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
-          n.phi.block <- length(phi.beg)
-          }else if(remainder==1)
-          {
-          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)    
-          }else
-          {
-          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)
-          }
-     }
+     }     
 
 
 ## Matrices to store samples
@@ -205,37 +202,12 @@ accept.all <- rep(0,6)
 accept <- accept.all
 proposal.sd.beta <- 0.01
 proposal.sd.phi <- 0.1
-proposal.sd.rho <- 0.02
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
 chol.proposal.corr.beta <- chol(proposal.corr.beta) 
-tau2.posterior.shape <- 0.5 * n - 1
-
-
-#### Priors
-## Put in default priors
-## N(0, 100) for beta 
-## U(0, 10) for tau2
-    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
-    if(is.null(prior.max.tau2)) prior.max.tau2 <- 1000
-
-    
-## Checks    
-    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
- 
-    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
-    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
-
-    if(length(prior.max.tau2)!=1) stop("the maximum prior value for tau2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.tau2)) stop("the maximum prior value for tau2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.tau2))!=0) stop("the maximum prior value for tau2 has missing values.", call.=FALSE)    
-    if(min(prior.max.tau2) <=0) stop("the maximum prior value for tau2 is less than zero", call.=FALSE)
-
-
+tau2.posterior.shape <- prior.tau2[1] + 0.5 * n
+rho.step <- 5
+      
+     
 #### CAR quantities
     if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
     if(nrow(W)!= n) stop("W has the wrong number of rows.", call.=FALSE)
@@ -246,60 +218,65 @@ tau2.posterior.shape <- 0.5 * n - 1
     if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
 
 
-## Create the precision matrix Q
+## Create the duplet form
 n.neighbours <- as.numeric(apply(W, 1, sum))
+W.duplet <- c(NA, NA)
+     for(i in 1:n)
+     {
+          for(j in 1:n)
+          {
+               if(W[i,j]==1)
+               {
+               W.duplet <- rbind(W.duplet, c(i,j))     
+               }else{}
+          }
+     }
+W.duplet <- W.duplet[-1, ]     
+n.duplet <- nrow(W.duplet) 
+
+
+## Create the list object
+Wlist <- as.list(rep(NA,n))     
+     for(i in 1:n)
+     {
+     Wlist[[i]] <- which(W[i, ]==1)     
+     }
+     
+## Create the set of determinants     
 Wstar <- diag(n.neighbours) - W
 Wstar.eigen <- eigen(Wstar)
-Wstar.val <- diag(Wstar.eigen$values)
-Wstar.vec <- Wstar.eigen$vectors
-Q <- rho * Wstar + (1-rho) * diag(rep(1,n))
-det.Q <- 0.5 * sum(log((rho * diag(Wstar.val) + (1-rho))))
-
-
-## Calculate eigen decomposition for blocks of Wstar               
-Wstar.list.vectors <- as.list(rep(0,n.phi.block))
-Wstar.list.values <- as.list(rep(0,n.phi.block))
-
-     for(i in 1:n.phi.block)
+Wstar.val <- Wstar.eigen$values
+n.rho <- length(prior.rho)
+det.Q <- rep(NA, n.rho)
+     for(i in 1:n.rho)
      {
-     temp <- eigen(Wstar[phi.beg[i]:phi.fin[i], phi.beg[i]:phi.fin[i]])     
-     Wstar.list.vectors[[i]] <- temp$vectors
-     Wstar.list.values[[i]] <- temp$values
+     det.Q[i] <-  0.5 * sum(log((prior.rho[i] * Wstar.val + (1-prior.rho[i]))))    
      }
 
 
-#### Other quantities needed for the MCMC algorithm
-failures <- trials - Y
-
 
 
 ###########################
 #### Run the Bayesian model
 ###########################
+## Start timer
+cat("Collecting", n.sample, "samples\n", sep = " ")
+progressBar <- txtProgressBar(style = 3)
+percentage.points<-round((1:100/100)*n.sample)
+     
     for(j in 1:n.sample)
     {
     ####################
     ## Sample from beta
     ####################
     proposal <- beta + (sqrt(proposal.sd.beta)* t(chol.proposal.corr.beta)) %*% rnorm(p)
-    proposal.beta <- beta    
-    phi.offset <- phi + offset
-        
-        for(r in 1:n.beta.block)
-        {
-        ## Propose a value
-        proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-        logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + phi.offset
-        logit.current <- as.numeric(X.standardised %*% beta) + phi.offset    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
+    proposal.beta <- beta
+    offset.temp <- phi + offset
 
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y * (log(prob.proposal) - log(prob.current)) + failures * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- sum(((beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2 - (proposal.beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2) / prior.var.beta[beta.beg[r]:beta.fin[r]])
-        prob <- exp(prob1 + prob2)
-
-        ## Accept or reject the value
+       for(r in 1:n.beta.block)
+       {
+       proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
+       prob <- binomialbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta)
             if(prob > runif(1))
             {
             beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -317,78 +294,47 @@ failures <- trials - Y
     ####################
     ## Sample from phi
     ####################
-    Q.temp <- Q / tau2
-    beta.offset <- as.numeric(X.standardised %*% beta) + offset        
-    b <- rnorm(n)
+    beta.offset <- X.standardised %*% beta + offset
+    temp1 <- binomialcarupdate(W_list=Wlist, nsites=n, phi=phi, nneighbours=n.neighbours, tau2=tau2, y=Y, failures=failures, phi_tune=proposal.sd.phi, rho_num=rho, rho_den=rho, offset=beta.offset)
+    phi <- temp1[[1]]
+    phi <- phi - mean(phi)
+    accept[3] <- accept[3] + temp1[[2]]
+    accept[4] <- accept[4] + n
+
+
          
-        for(r in 1:n.phi.block)
-        {
-        ## Propose a value
-        Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
-        lambda <- rho *  Wstar.list.values[[r]] + (1-rho)  
-        block.var <- tau2 * Wstar.list.vectors[[r]]  %*% diag(1/lambda) %*% t(Wstar.list.vectors[[r]])
-        block.mean <- - block.var %*% (Q.temp[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])] %*% phi[-(phi.beg[r]:phi.fin[r])])
-        proposal.phi <- phi[phi.beg[r]:phi.fin[r]] + (sqrt(proposal.sd.phi) * t(chol(block.var))) %*% b[phi.beg[r]:phi.fin[r]]             
-        logit.proposal <- beta.offset[phi.beg[r]:phi.fin[r]] + proposal.phi
-        logit.current <- beta.offset[phi.beg[r]:phi.fin[r]] + phi[phi.beg[r]:phi.fin[r]]    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-        
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y[phi.beg[r]:phi.fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[phi.beg[r]:phi.fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- t(phi[phi.beg[r]:phi.fin[r]] - block.mean) %*% Q.current %*% (phi[phi.beg[r]:phi.fin[r]] - block.mean) - t(proposal.phi - block.mean) %*% Q.current %*% (proposal.phi - block.mean)
-        prob <- exp(prob1 + 0.5 * prob2)
-
-        ## Accept or reject the value
-            if(prob > runif(1))
-            {
-            phi[phi.beg[r]:phi.fin[r]] <- proposal.phi
-            accept[3] <- accept[3] + 1  
-            }else
-            {
-            }
-        }                      
-     accept[4] <- accept[4] + n.phi.block
-     phi <- phi - mean(phi)
-
-
-
     ##################
     ## Sample from tau2
     ##################
-    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
-    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
-    
-    
+    temp2 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=rho, offdiagonal=rho)      
+    tau2.posterior.scale <- temp2 + prior.tau2[2] 
+    tau2 <- 1 / rgamma(1, tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
+         
+
     ##################
     ## Sample from rho
     ##################
-    #### Propose a value
-    proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1,  mean=rho, sd=proposal.sd.rho)    
-    
-    #### Calculate the acceptance probability
-    proposal.Q <- -proposal.rho * W 
-    diag(proposal.Q) <- n.neighbours * proposal.rho + 1 - proposal.rho
-    proposal.det.Q <- 0.5 * sum(log((proposal.rho * diag(Wstar.val) + (1-proposal.rho))))
-    logprob.current <- det.Q - tau2.posterior.scale / tau2
-    logprob.proposal <- proposal.det.Q - 0.5 * sum(phi * (proposal.Q %*% phi)) / tau2
+    which.proposal.rho <- sample(setdiff(max(1, which.rho-rho.step):min(which.rho+rho.step, n.rho), which.rho), size=1)    
+    proposal.rho <- prior.rho[which.proposal.rho]
+    temp3 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=proposal.rho, offdiagonal=proposal.rho)      
+    logprob.current <- det.Q[which.rho] - temp2 / tau2
+    logprob.proposal <- det.Q[which.proposal.rho] - temp3 / tau2
     prob <- exp(logprob.proposal - logprob.current)
     
     #### Accept or reject the proposal
          if(prob > runif(1))
          {
          rho <- proposal.rho
-         Q <- proposal.Q
-         det.Q <- proposal.det.Q        
+         which.rho <- which.proposal.rho
          accept[5] <- accept[5] + 1           
          }else
          {
          }              
     accept[6] <- accept[6] + 1           
-    
 
    
+         
     #########################
     ## Calculate the deviance
     #########################
@@ -452,10 +398,10 @@ failures <- trials - Y
        #### rho tuning parameter
                if(accept.rho > 70)
                {
-               proposal.sd.rho <- min(2 * proposal.sd.rho, 10)
+               rho.step <- min(rho.step+1, round(n.rho/4))
                }else if(accept.rho < 50)              
                {
-               proposal.sd.rho <- 0.5 * proposal.sd.rho
+               proposal.sd.rho <- max(rho.step-1, 1)
                }else
                {
                }
@@ -465,21 +411,18 @@ failures <- trials - Y
 
     
     
-    #######################################
-    #### Print out the number of iterations
-    #######################################
-    k <- j/1000
-        if(ceiling(k)==floor(k))
-        {
-        cat("Completed ",j, " samples\n")
-        flush.console()
-        }else
-        {
-        }
-}
-
-
+    ################################       
+    ## print progress to the console
+    ################################
+          if(j %in% percentage.points)
+          {
+          setTxtProgressBar(progressBar, j/n.sample)
+          }
+     }
 
+# end timer
+cat("\nSummarising results")
+close(progressBar)
 ###################################
 #### Summarise and save the results 
 ###################################
@@ -502,7 +445,20 @@ p.d <- mean(samples.deviance) - deviance.fitted
 DIC <- 2 * mean(samples.deviance) - deviance.fitted
 
 
+#### Compute the Conditional Predictive Ordinate
+CPO.temp <- array(NA, c(nrow(samples.phi), n))
+    for(i in 1:nrow(samples.phi))
+    {
+    temp.logit <- samples.phi[i, ]  + X.standardised %*% samples.beta[i, ] + offset
+    temp <- exp(temp.logit)  / (1 + exp(temp.logit))
+    CPO.temp[i, ] <- 1 / dbinom(x=Y, size=trials, prob=temp)
+    }
+CPO <- 1/apply(CPO.temp, 2, mean)
+MPL <- sum(log(CPO))     
 
+     
+     
+     
 #### transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- samples.beta
 number.cts <- sum(X.indicator==1)     
@@ -594,8 +550,10 @@ residuals <- round(residuals, 4)
 ## Compile and return the results
 model.string <- c("Likelihood model - Binomial (logit link function)", "\nRandom effects model - Leroux CAR\n")
 samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), rho=mcmc(samples.rho))
-results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, summary.results=summary.results, model=model.string, accept=accept.final)
+results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, MPL=MPL, summary.results=summary.results, model=model.string, accept=accept.final)
 class(results) <- "carbayes"
+b<-proc.time()
+cat(" finished in ", round(b[3]-a[3], 1), "seconds")
 return(results)
 }
 
