@@ -1,6 +1,8 @@
 gaussian.bymCAR <-
-function(formula, data=NULL, beta=NULL, phi=NULL, theta=NULL, nu2=NULL, tau2=NULL, sigma2=NULL, W, burnin=0, n.sample=1000, thin=1, blocksize.phi=10, blocksize.theta=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.nu2=NULL, prior.max.tau2=NULL, prior.max.sigma2=NULL)
+function(formula, data=NULL, beta=NULL, phi=NULL, theta=NULL, nu2=NULL, tau2=NULL, sigma2=NULL, W, burnin=0, n.sample=1000, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL, prior.sigma2=NULL)
 {
+cat("Setting up the model\n")
+a<-proc.time()
 ##############################################
 #### Format the arguments and check for errors
 ##############################################
@@ -119,7 +121,37 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(sigma2 <= 0) stop("sigma2 is negative or zero.", call.=FALSE)
 
 
+#### Priors
+## Put in default priors
+    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.tau2)) prior.tau2 <- c(0.001, 0.001)
+    if(is.null(prior.nu2)) prior.nu2 <- c(0.001, 0.001)
+    if(is.null(prior.sigma2)) prior.sigma2 <- c(0.001, 0.001)        
+        
+## Checks    
+    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
+ 
+    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
+    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
 
+    if(length(prior.tau2)!=2) stop("the prior value for tau2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.tau2)) stop("the prior value for tau2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.tau2))!=0) stop("the prior value for tau2 has missing values.", call.=FALSE)    
+
+    if(length(prior.nu2)!=2) stop("the prior value for nu2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.nu2)) stop("the prior value for nu2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.nu2))!=0) stop("the prior value for nu2 has missing values.", call.=FALSE)    
+
+    if(length(prior.sigma2)!=2) stop("the prior value for sigma2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.sigma2)) stop("the prior value for sigma2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.sigma2))!=0) stop("the prior value for sigma2 has missing values.", call.=FALSE)    
+
+     
 #### MCMC quantities
 ## Checks
     if(!is.numeric(burnin)) stop("burn-in is not a number", call.=FALSE)
@@ -130,42 +162,6 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(!is.numeric(thin)) stop("thin is not a number", call.=FALSE)
     if(thin <= 0) stop("thin is less than or equal to zero.", call.=FALSE)
 
-    if(!is.numeric(blocksize.phi)) stop("blocksize.phi is not a number", call.=FALSE)
-    if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
-    if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
-    if(!is.numeric(blocksize.theta)) stop("blocksize.theta is not a number", call.=FALSE)
-    if(blocksize.theta <= 0) stop("blocksize.theta is less than or equal to zero", call.=FALSE)
-    if(!(floor(blocksize.theta)==ceiling(blocksize.theta))) stop("blocksize.theta has non-integer values.", call.=FALSE)
-
-
-## Compute the blocking structure for phi
-     if(blocksize.phi >= n)
-     {
-     n.phi.block <- 1
-     phi.beg <- 1
-     phi.fin <- n  
-     }else
-     {
-     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
-     remainder <- n - (n.standard * blocksize.phi)
-     
-          if(remainder==0)
-          {
-          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
-          n.phi.block <- length(phi.beg)
-          }else if(remainder==1)
-          {
-          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)    
-          }else
-          {
-          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)
-          }
-     }
 
 
 ## Matrices to store samples
@@ -179,48 +175,11 @@ samples.sigma2 <- array(NA, c(n.keep, 1))
 samples.deviance <- array(NA, c(n.keep, 1))
 
 ## Metropolis quantities
-nu2.posterior.shape <- 0.5*n-1
-tau2.posterior.shape <- 0.5*n-1
-sigma2.posterior.shape <- 0.5 * n - 1
-
-
-#### Priors
-## Put in default priors
-## N(0, 100) for beta 
-## U(0, 10) for variance parameters
-    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
-    if(is.null(prior.max.tau2)) prior.max.tau2 <- 1000
-    if(is.null(prior.max.sigma2)) prior.max.sigma2 <- 1000
-    if(is.null(prior.max.nu2)) prior.max.nu2 <- 1000
-        
-        
-## Checks    
-    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
- 
-    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
-    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
-
-    if(length(prior.max.tau2)!=1) stop("the maximum prior value for tau2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.tau2)) stop("the maximum prior value for tau2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.tau2))!=0) stop("the maximum prior value for tau2 has missing values.", call.=FALSE)    
-    if(min(prior.max.tau2) <=0) stop("the maximum prior value for tau2 is less than zero", call.=FALSE)
-    
-    if(length(prior.max.sigma2)!=1) stop("the maximum prior value for sigma2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.sigma2)) stop("the maximum prior value for sigma2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.sigma2))!=0) stop("the maximum prior value for sigma2 has missing values.", call.=FALSE)    
-    if(min(prior.max.sigma2) <=0) stop("the maximum prior value for sigma2 is less than zero", call.=FALSE)
-
-    if(length(prior.max.nu2)!=1) stop("the maximum prior value for nu2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.nu2)) stop("the maximum prior value for nu2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.nu2))!=0) stop("the maximum prior value for nu2 has missing values.", call.=FALSE)    
-    if(min(prior.max.nu2) <=0) stop("the maximum prior value for nu2 is less than zero", call.=FALSE)
-
-
+sigma2.posterior.shape <- prior.sigma2[1] + 0.5*n
+tau2.posterior.shape <- prior.tau2[1] + 0.5*(n-1)
+nu2.posterior.shape <- prior.nu2[1] + 0.5*n
+     
+     
 #### CAR quantities
     if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
     if(nrow(W)!= n) stop("W has the wrong number of rows.", call.=FALSE)
@@ -230,18 +189,28 @@ sigma2.posterior.shape <- 0.5 * n - 1
     if(min(W)<0) stop("W has negative elements.", call.=FALSE)
     if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
 
+### Create the duplet form
 n.neighbours <- as.numeric(apply(W, 1, sum))
-Q <- diag(n.neighbours)  - W
-
-
-## quantities required in updating phi              
-block.mean.part <- as.list(rep(0,n.phi.block))
-
-     for(r in 1:n.phi.block)
+W.duplet <- c(NA, NA)
+     for(i in 1:n)
      {
-     Q.current <- Q[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
-     block.var <- chol2inv(chol(Q.current))
-     block.mean.part[[r]] <- - block.var %*% Q[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])]
+          for(j in 1:n)
+          {
+               if(W[i,j]==1)
+               {
+               W.duplet <- rbind(W.duplet, c(i,j))     
+               }else{}
+          }
+     }
+W.duplet <- W.duplet[-1, ]     
+n.duplet <- nrow(W.duplet) 
+
+
+## Create the list object
+Wlist <- as.list(rep(NA,n))     
+     for(i in 1:n)
+     {
+     Wlist[[i]] <- which(W[i, ]==1)     
      }
 
 
@@ -261,6 +230,11 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
 ###########################
 #### Run the Bayesian model
 ###########################
+## Start timer
+cat("Collecting", n.sample, "samples\n", sep = " ")
+progressBar <- txtProgressBar(style = 3)
+percentage.points<-round((1:100/100)*n.sample)
+     
     for(j in 1:n.sample)
     {
     ####################
@@ -278,35 +252,16 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
     ## Sample from nu2
     ##################
     fitted.current <-  as.numeric(X.standardised %*% beta) + phi + theta + offset
-    nu2.posterior.scale <- 0.5 * sum((Y - fitted.current)^2)
-    nu2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.nu2), b=Inf,  shape=nu2.posterior.shape, scale=(1/nu2.posterior.scale))
-    
+    nu2.posterior.scale <- prior.nu2[2] + 0.5 * sum((Y - fitted.current)^2)
+    nu2 <- 1 / rgamma(1, nu2.posterior.shape, scale=(1/nu2.posterior.scale))    
 
 
     ####################
     ## Sample from phi
     ####################
-    Q.temp <- Q / tau2
-    data.mean.phi <- Y - as.numeric(X.standardised %*% beta) - theta - offset    
-    data.precision.phi <- diag(rep((1/nu2), blocksize.phi))
-    b <- rnorm(n)
-    
-         for(r in 1:n.phi.block)
-         {
-         ## Create the prior  mean and variance
-         n.current <- length(phi.beg[r]:phi.fin[r])
-         Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
-         block.mean <- block.mean.part[[r]] %*% phi[-(phi.beg[r]:phi.fin[r])]
-         
-         ## Create the full conditional
-         U <- chol(Q.current + data.precision.phi[1:n.current, 1:n.current])
-         Uinv <- backsolve(U, diag(rep(1,n.current)))
-         fc.mean.phi <- (Uinv %*% t(Uinv)) %*% (Q.current %*% block.mean + data.precision.phi[1:n.current, 1:n.current] %*% data.mean.phi[phi.beg[r]:phi.fin[r]])
-         
-         ## Update phi
-         phi[phi.beg[r]:phi.fin[r]] <- fc.mean.phi + Uinv %*% b[phi.beg[r]:phi.fin[r]]
-         }   
-    phi <- phi - mean(phi)            
+    offset.phi <- (Y - as.numeric(X.standardised %*% beta) - theta - offset) / nu2    
+    phi <- gaussiancarupdate(W_list=Wlist, nsites=n, phi=phi, nneighbours=n.neighbours, tau2=tau2, rho_num=1, rho_den=1, nu2=nu2, offset=offset.phi)
+    phi <- phi - mean(phi)
     
 
     
@@ -324,16 +279,17 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
     ###################
     ## Sample from tau2
     ###################
-    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
-    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
+    temp2 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=1, offdiagonal=1)      
+    tau2.posterior.scale <- temp2 + prior.tau2[2] 
+    tau2 <- 1 / rgamma(1, tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
     
     
     #####################
     ## Sample from sigma2
     #####################
-    sigma2.posterior.scale <- 0.5*sum(theta^2)
-    sigma2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.sigma2), b=Inf,  shape=sigma2.posterior.shape, scale=(1/sigma2.posterior.scale))
+    sigma2.posterior.scale <- prior.sigma2[2] + 0.5*sum(theta^2)
+    sigma2 <- 1 / rgamma(1, sigma2.posterior.shape, scale=(1/sigma2.posterior.scale))
     
     
     
@@ -364,28 +320,28 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
         
     
     
-    #######################################
-    #### Print out the number of iterations
-    #######################################
-    k <- j/1000
-        if(ceiling(k)==floor(k))
-        {
-        cat("Completed ",j, " samples\n")
-        flush.console()
-        }else
-        {
-        }
-}
+    ################################       
+    ## print progress to the console
+    ################################
+          if(j %in% percentage.points)
+          {
+          setTxtProgressBar(progressBar, j/n.sample)
+          }
+     }
 
-
-accept.final <- rep(100, 6)
-names(accept.final) <- c("beta", "phi", "nu2", "tau2", "theta", "sigma")
-
-     
+# end timer
+cat("\nSummarising results")
+close(progressBar)
+  
      
 ###################################
 #### Summarise and save the results 
 ###################################
+## Compute the acceptance rates
+accept.final <- rep(100, 6)
+names(accept.final) <- c("beta", "phi", "nu2", "tau2", "theta", "sigma")
+     
+     
 ## Deviance information criterion (DIC)
 median.beta <- apply(samples.beta, 2, median)
 median.phi <- apply(samples.phi, 2, median)
@@ -397,7 +353,17 @@ p.d <- mean(samples.deviance) - deviance.fitted
 DIC <- 2 * mean(samples.deviance) - deviance.fitted
 
 
-
+#### Compute the Conditional Predictive Ordinate
+CPO.temp <- array(NA, c(nrow(samples.theta), n))
+    for(i in 1:nrow(samples.theta))
+    {
+    temp.fitted <- samples.theta[i, ] + samples.phi[i, ] + X.standardised %*% samples.beta[i, ] + offset
+    CPO.temp[i, ] <- 1 / dnorm(x=Y, mean=temp.fitted, sd=sqrt(samples.nu2[i,1]))
+    }
+CPO <- 1/apply(CPO.temp, 2, mean)
+MPL <- sum(log(CPO)) 
+     
+     
 #### transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- samples.beta
 number.cts <- sum(X.indicator==1)     
@@ -489,8 +455,10 @@ residuals <- round(residuals, 4)
 ## Compile and return the results
 model.string <- c("Likelihood model - Gaussian (identity link function)", "\nRandom effects model - BYM CAR\n")     
 samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), theta=mcmc(samples.theta), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), sigma2=mcmc(samples.sigma2))
-results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, summary.results=summary.results, model=model.string, accept=accept.final)
+results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, MPL=MPL, summary.results=summary.results, model=model.string, accept=accept.final)
 class(results) <- "carbayes"
+b<-proc.time()
+cat(" finished in ", round(b[3]-a[3], 1), "seconds")
 return(results)
 }
 
