@@ -1,6 +1,8 @@
 poisson.iarCAR <-
-function(formula, data=NULL, beta=NULL, phi=NULL, tau2=NULL, W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, blocksize.phi=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.tau2=NULL)
+function(formula, data=NULL, beta=NULL, phi=NULL, tau2=NULL, W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL)
 {
+cat("Setting up the model\n")
+a<-proc.time()
 ##############################################
 #### Format the arguments and check for errors
 ##############################################
@@ -102,6 +104,29 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(tau2 <= 0) stop("tau2 is negative or zero.", call.=FALSE)
 
 
+#### Priors
+## Put in default priors
+    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.tau2)) prior.tau2 <- c(0.001, 0.001)
+
+    
+## Checks    
+    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
+ 
+    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
+    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
+
+    if(length(prior.tau2)!=2) stop("the prior value for tau2 is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.tau2)) stop("the prior value for tau2 is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.tau2))!=0) stop("the prior value for tau2 has missing values.", call.=FALSE)    
+
+     
+     
 #### MCMC quantities
 ## Checks
     if(!is.numeric(burnin)) stop("burn-in is not a number", call.=FALSE)
@@ -115,12 +140,9 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.beta)==ceiling(blocksize.beta))) stop("blocksize.beta has non-integer values.", call.=FALSE)
-    if(!is.numeric(blocksize.phi)) stop("blocksize.phi is not a number", call.=FALSE)
-    if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
-    if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
 
 
-## Compute the blocking structure for beta
+## Compute the blocking structure for beta     
      if(blocksize.beta >= p)
      {
      n.beta.block <- 1
@@ -134,47 +156,17 @@ offset <- try(model.offset(frame), silent=TRUE)
           if(remainder==0)
           {
           beta.beg <- c(1,seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta))
+          beta.fin <- seq(blocksize.beta, p, blocksize.beta)
           n.beta.block <- length(beta.beg)
           }else
           {
           beta.beg <- c(1, seq((blocksize.beta+1), p, blocksize.beta))
-          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta), p)
+          beta.fin <- c(seq((blocksize.beta), p, blocksize.beta), p)
           n.beta.block <- length(beta.beg)
           }
      }         
 
 
-## Compute the blocking structure for phi
-     if(blocksize.phi >= n)
-     {
-     n.phi.block <- 1
-     phi.beg <- 1
-     phi.fin <- n  
-     }else
-     {
-     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
-     remainder <- n - (n.standard * blocksize.phi)
-     
-          if(remainder==0)
-          {
-          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
-          n.phi.block <- length(phi.beg)
-          }else if(remainder==1)
-          {
-          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)    
-          }else
-          {
-          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
-          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
-          n.phi.block <- length(phi.beg)
-          }
-     }
-
-
 
 ## Matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
@@ -190,32 +182,7 @@ proposal.sd.beta <- 0.01
 proposal.sd.phi <- 0.1
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
 chol.proposal.corr.beta <- chol(proposal.corr.beta) 
-tau2.posterior.shape <- 0.5 * n - 1
-
-
-#### Priors
-## Put in default priors
-## N(0, 100) for beta 
-## U(0, 10) for tau2
-    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
-    if(is.null(prior.max.tau2)) prior.max.tau2 <- 1000
-
-    
-## Checks    
-    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)    
- 
-    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
-    if(min(prior.var.beta) <=0) stop("the vector of prior variances has elements less than zero", call.=FALSE)
-
-    if(length(prior.max.tau2)!=1) stop("the maximum prior value for tau2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.max.tau2)) stop("the maximum prior value for tau2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.max.tau2))!=0) stop("the maximum prior value for tau2 has missing values.", call.=FALSE)    
-    if(min(prior.max.tau2) <=0) stop("the maximum prior value for tau2 is less than zero", call.=FALSE)
+tau2.posterior.shape <- prior.tau2[1] + 0.5 * (n-1)
 
 
 #### CAR quantities
@@ -227,97 +194,84 @@ tau2.posterior.shape <- 0.5 * n - 1
     if(min(W)<0) stop("W has negative elements.", call.=FALSE)
     if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
 
+## Create the duplet form
 n.neighbours <- as.numeric(apply(W, 1, sum))
-Q <- diag(n.neighbours)  - W
-
-## quantities required in updating phi              
-block.mean.part <- as.list(rep(0,n.phi.block))
-block.var.chol <- as.list(rep(0,n.phi.block))
-
-     for(r in 1:n.phi.block)
+W.duplet <- c(NA, NA)
+     for(i in 1:n)
      {
-     Q.current <- Q[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
-     block.var <- chol2inv(chol(Q.current))
-     block.mean.part[[r]] <- - block.var %*% Q[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])]
-     block.var.chol[[r]] <- chol(block.var)
+          for(j in 1:n)
+          {
+               if(W[i,j]==1)
+               {
+               W.duplet <- rbind(W.duplet, c(i,j))     
+               }else{}
+          }
      }
+W.duplet <- W.duplet[-1, ]     
+n.duplet <- nrow(W.duplet) 
 
 
+## Create the list object
+Wlist <- as.list(rep(NA,n))     
+     for(i in 1:n)
+     {
+     Wlist[[i]] <- which(W[i, ]==1)     
+     }
+
 
 ###########################
 #### Run the Bayesian model
 ###########################
+## Start timer
+cat("Collecting", n.sample, "samples\n", sep = " ")
+progressBar <- txtProgressBar(style = 3)
+percentage.points<-round((1:100/100)*n.sample)
+     
     for(j in 1:n.sample)
     {
     ####################
     ## Sample from beta
     ####################
     proposal <- beta + (sqrt(proposal.sd.beta)* t(chol.proposal.corr.beta)) %*% rnorm(p)
-    proposal.beta <- beta    
-    phi.offset <- exp(phi + offset)
-                  
-          for(r in 1:n.beta.block)
-          {
-          ## Calculate the acceptance probability          
-          proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-          lp.proposal <- as.numeric(X.standardised %*% proposal.beta)
-          lp.current <- as.numeric(X.standardised %*% beta)
-          prob1 <- sum(Y * (lp.proposal - lp.current) + phi.offset * (exp(lp.current) - exp(lp.proposal)))
-          prob2 <- sum(((beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2 - (proposal.beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2) / prior.var.beta[beta.beg[r]:beta.fin[r]])
-          prob <- exp(prob1 + 0.5 * prob2)
-              
-          ## Accept or reject the value
-              if(prob > runif(1))
-              {
-              beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
-              accept[1] <- accept[1] + 1  
-              }else
-              {
-              proposal.beta[beta.beg[r]:beta.fin[r]] <- beta[beta.beg[r]:beta.fin[r]]
-              }
-         }
-    accept[2] <- accept[2] + n.beta.block
+    proposal.beta <- beta
+    offset.temp <- phi + offset
+
+       for(r in 1:n.beta.block)
+       {
+       proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
+       prob <- poissonbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y, prior.mean.beta, prior.var.beta)
+            if(prob > runif(1))
+            {
+            beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
+            accept[1] <- accept[1] + 1  
+            }else
+            {
+            proposal.beta[beta.beg[r]:beta.fin[r]] <- beta[beta.beg[r]:beta.fin[r]]
+            }
+        }
+
+    accept[2] <- accept[2] + n.beta.block    
          
 
 
     ####################
     ## Sample from phi
     ####################
-    Q.temp <- Q / tau2
-    beta.offset <- exp(as.numeric(X.standardised %*% beta) + offset)        
-    b <- rnorm(n)
-    
-         for(r in 1:n.phi.block)   
-         {
-         ## Propose a value
-         Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
-         block.mean <- block.mean.part[[r]] %*% phi[-(phi.beg[r]:phi.fin[r])]
-         proposal.phi <- phi[phi.beg[r]:phi.fin[r]] + (sqrt(proposal.sd.phi) * sqrt(tau2) * t(block.var.chol[[r]])) %*% b[phi.beg[r]:phi.fin[r]]
-
-         ## Calculate the acceptance probability
-         prob1 <- sum(Y[phi.beg[r]:phi.fin[r]] * (proposal.phi - phi[phi.beg[r]:phi.fin[r]]) + beta.offset[phi.beg[r]:phi.fin[r]] * (exp(phi[phi.beg[r]:phi.fin[r]]) - exp(proposal.phi)))
-         prob2 <- t(phi[phi.beg[r]:phi.fin[r]] - block.mean) %*% Q.current %*% (phi[phi.beg[r]:phi.fin[r]] - block.mean) - t(proposal.phi - block.mean) %*% Q.current %*% (proposal.phi - block.mean)
-         prob <- exp(prob1 + 0.5 * prob2)
-         
-         ## Accept or reject the value
-              if(prob > runif(1))
-              {
-              phi[phi.beg[r]:phi.fin[r]] <- proposal.phi
-              accept[3] <- accept[3] + 1  
-              }else
-              {
-              }
-        }                
-    accept[4] <- accept[4] + n.phi.block
-    phi <- phi - mean(phi)              
+    beta.offset <- X.standardised %*% beta + offset
+    temp1 <- poissoncarupdate(W_list=Wlist, nsites=n, phi=phi, nneighbours=n.neighbours, tau2=tau2, y=Y, phi_tune=proposal.sd.phi, rho_num=1, rho_den=1, offset=beta.offset)
+    phi <- temp1[[1]]
+    phi <- phi - mean(phi)
+    accept[3] <- accept[3] + temp1[[2]]
+    accept[4] <- accept[4] + n    
     
     
 
     ##################
     ## Sample from tau2
     ##################
-    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
-    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
+    temp2 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=1, offdiagonal=1)      
+    tau2.posterior.scale <- temp2 + prior.tau2[2] 
+    tau2 <- 1 / rgamma(1, tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
             
     
@@ -383,21 +337,19 @@ block.var.chol <- as.list(rep(0,n.phi.block))
 
     
     
-    #######################################
-    #### Print out the number of iterations
-    #######################################
-    k <- j/1000
-        if(ceiling(k)==floor(k))
-        {
-        cat("Completed ",j, " samples\n")
-        flush.console()
-        }else
-        {
-        }
-}
-
-
+    ################################       
+    ## print progress to the console
+    ################################
+          if(j %in% percentage.points)
+          {
+          setTxtProgressBar(progressBar, j/n.sample)
+          }
+     }
 
+# end timer
+cat("\nSummarising results")
+close(progressBar)
+     
 ###################################
 #### Summarise and save the results 
 ###################################
@@ -417,8 +369,19 @@ deviance.fitted <- -2 * sum(Y * log(fitted.median) -  fitted.median - lfactorial
 p.d <- mean(samples.deviance) - deviance.fitted
 DIC <- 2 * mean(samples.deviance) - deviance.fitted
 
+     
+#### Compute the Conditional Predictive Ordinate
+CPO.temp <- array(NA, c(nrow(samples.phi), n))
+    for(i in 1:nrow(samples.phi))
+    {
+    temp.lp <- samples.phi[i, ] + X.standardised %*% samples.beta[i, ] + offset
+    temp.fitted <- exp(temp.lp)
+    CPO.temp[i, ] <- 1 / dpois(x=Y, lambda=temp.fitted)
+    }
+CPO <- 1/apply(CPO.temp, 2, mean)
+MPL <- sum(log(CPO)) 
 
-
+     
 #### transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- samples.beta
 number.cts <- sum(X.indicator==1)     
@@ -501,8 +464,10 @@ residuals <- round(residuals, 4)
 
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - Intrinsic CAR\n")
 samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2))
-results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, summary.results=summary.results, model=model.string, accept=accept.final)
+results <- list(formula=formula, samples=samples, fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, W.summary=W, DIC=DIC, p.d=p.d, MPL=MPL, summary.results=summary.results, model=model.string, accept=accept.final)
 class(results) <- "carbayes"
+b<-proc.time()
+cat(" finished in ", round(b[3]-a[3], 1), "seconds")
 return(results)
 }
 
